/* 
 * keyshades
 * a tool for disabling the treo keyboard backlight
 * copyright (c) 2005, daniel grobe sachs <dgsachs@nekito.net>
 *
 * This is free software. See included LICENSE file for details.
 */

#include <PalmOS.h>
#include <palmOne_68K.h>
#include <68K/System/HsExt.h>
#include <Common/System/HsExtCommon.h>

#include "resource.h"

#define FTR_LIGHTSON 0
#define FTR_SIDEHELD 1

#define DefaultLit false	// Default to lights out
        //hsLightCircumstanceKeylightOff          = 4,    /* turn keylight off */


static void RegisterForNotifications()
{
	UInt16 cardNo;
	LocalID dbID;

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	FtrSet(CREATOR,FTR_LIGHTSON,false); // We start up with the lights on
	FtrSet(CREATOR,FTR_SIDEHELD,0);

	if( !DefaultLit )
	{
		HsLightCircumstance(true,hsLightCircumstanceKeylightOff);
	}

	SysNotifyRegister(cardNo, dbID, sysNotifyEventDequeuedEvent, NULL,
			sysNotifyNormalPriority, 0);
}

static void DeregisterNotifications()
{
	UInt16 cardNo;
	LocalID dbID;

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent,
			sysNotifyNormalPriority);
}

#define EndianSwap16(n) (((((UInt16) n) << 8) & 0xFF00) | \
		         ((((UInt16) n) >> 8) & 0x00FF))

static void HandleNotification(SysNotifyParamType *np)
{
	UInt32 mode;
	EventType *eventP = (EventType*)np->notifyDetailsP;

	// Is this a queue event?
	if( np->notifyType != sysNotifyEventDequeuedEvent )
		return;

	// Yes, fiddle!

	switch( EndianSwap16(eventP->eType) )
	{
		case keyDownEvent:
			switch( EndianSwap16(eventP->data.keyDown.chr) )
			{
				case 0x161F: // Side

					// Check if we've held it
					if( EndianSwap16(eventP->data.keyDown.modifiers) 
							!= 0x848 )
					{
						FtrSet(CREATOR,FTR_SIDEHELD,false);
						break;
					}

					// Check if we've already changed it
					FtrGet(CREATOR,FTR_SIDEHELD,&mode);
					if( mode )
						break;

					// Toggle
					FtrGet(CREATOR,FTR_LIGHTSON,&mode);
					HsLightCircumstance(mode,hsLightCircumstanceKeylightOff);

					// Save result
					FtrSet(CREATOR,FTR_LIGHTSON,!mode);

					// Set debouncer
					FtrSet(CREATOR,FTR_SIDEHELD,true);

					break;

				default:
					break;
			}
			break;

		default:
			break;
	}
}

//// PilotMain /////////////////////////////////////////////////////////
// It all starts here...

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) 
{
	Err err = 0;
	EventType event;

	switch( launchCode ) 
	{
		case sysAppLaunchCmdSystemReset :
			RegisterForNotifications();
			break;

		case sysAppLaunchCmdNotify : 
			HandleNotification((SysNotifyParamType *)cmdPBP);
			break;

		case sysAppLaunchCmdNormalLaunch :
			// Well, uh, we don't have a UI, so we just quit
			MemSet( &event, sizeof(event), 0);

			event.eType = appStopEvent;
			EvtAddEventToQueue(&event);
			break;

		default:
			break;
	}


	return err;
}

